@@ -1,8 +1,9 @@
-from .main import parse
+from .main import parse, parse_html
 from .paragraph import Paragraph, Pos
 
 __all__ = [
     "parse",
+    "parse_html",
     "Paragraph",
     "Pos",
 ]
