@@ -1,5 +1,5 @@
 from dataclasses import dataclass
-from typing import Iterable, Optional
+from typing import Iterable
 
 from .syntax import FENCE_END, FENCE_START, HEAD_REGEX, JA_REGEX, LIST_REGEX
 
@@ -25,9 +25,15 @@ class IgnoreBlock:
 
 @dataclass
 class Line:
+    """Annotation for each body lines.
+    japick will clean all of syntax of original text,
+    so before the process, We need to annotate information.
+    """
+
     body: str
     is_head: bool = False
     is_list: bool = False
+    is_ignore: bool = False
 
     def __eq__(self, other) -> bool:
         if other is None:
@@ -36,18 +42,19 @@ class Line:
             self.body == other.body
             and self.is_head is other.is_head
             and self.is_list is other.is_list
+            and self.is_ignore is other.is_ignore
         )
 
 
-def parse_lines(body: str) -> Iterable[Optional[Line]]:
+def parse_lines(body: str) -> Iterable[Line]:
     lines = body.splitlines()
     is_ignore = IgnoreBlock()
 
     for line in lines:
         if is_ignore.should_ignore(line):
-            yield None
+            yield Line(line, is_ignore=True)
         elif JA_REGEX.search(line) is None:
-            yield None
+            yield Line(line, is_ignore=True)
         elif HEAD_REGEX.search(line):
             yield Line(line, is_head=True)
         elif LIST_REGEX.search(line):
